import {
  AfterContentInit,
  AfterViewInit,
  Component,
  EventEmitter,
  Input,
  OnChanges,
  OnInit,
  Output,
  SimpleChanges,
  ViewChild
} from '@angular/core';
import {
  AdvancedScheduleFieldInfo, AdvancedScheduleInfo,
  ScheduleInfo,
  ScheduleService,
  TaskEntityInfo,
  TaskgroupEntityInfo
} from "../../../api";
import {DateTimePickerComponent} from "../../date-time-picker/date-time-picker.component";
import * as moment from "moment";

@Component({
  selector: 'app-advanced-scheduler',
  templateUrl: './advanced-scheduler.component.html',
  styleUrls: ['./advanced-scheduler.component.css']
})
export class AdvancedSchedulerComponent implements OnInit, OnChanges{
  @Input() task: TaskEntityInfo | undefined;
  @Input() taskgroup: TaskgroupEntityInfo | undefined
  @Input() scheduleInfo: ScheduleInfo | undefined

  @ViewChild('startTimePicker') startTimePicker?: DateTimePickerComponent;
  @ViewChild('stopTimePicker') stopTimePicker?: DateTimePickerComponent;
  slideMinutes: number = 0;

  @Output('onStartTimeSet') startTimeEmitter : EventEmitter<Date> = new EventEmitter<Date>();
  @Output('onEndTimeSet') endTimeEmitter : EventEmitter<Date> = new EventEmitter<Date>();
  @Output('onSchedule') onSchedule: EventEmitter<AdvancedScheduleInfo> = new EventEmitter<AdvancedScheduleInfo>();
  selectedStartTime: Date | undefined
  selectedStopTime: Date | undefined

  currentProgress: string = "30%";
  futureProgress: string = "60%"

  constructor(private scheduleService: ScheduleService) {

  }

  ngOnInit() {
    this.calcFutureProgress();
    this.calcCurrentProgress();

    if(this.scheduleInfo != undefined) {
      const schedule = this.scheduleInfo as AdvancedScheduleInfo
      this.selectedStartTime = new Date(schedule.scheduleStartTime);
      this.selectedStopTime = new Date(schedule.scheduleStopTime);
      this.calcFutureProgress();
      this.calcCurrentProgress();
    }
  }

  ngOnChanges() {

  }

  setDate(clickedDate: Date) {
    if(this.selectedStartTime == undefined) {
      this.startTimePicker!.setDateTime(clickedDate);
      this.selectedStartTime = clickedDate;
    } else {
      this.stopTimePicker!.setDateTime(clickedDate);
      this.selectedStopTime = clickedDate;
    }

    this.calcFutureProgress();
  }

  setStopTime(selectedDate: Date) {
    this.selectedStopTime = selectedDate
    this.stopTimePicker!.setDateTime(selectedDate);

    this.calcFutureProgress();
  }
  setStartTime(selectedDate: Date) {
    this.selectedStartTime = selectedDate;
    this.startTimePicker!.setDateTime(selectedDate);


    this.calcFutureProgress();
  }

  onStartTimeSet(selectedDate: Date) {
    this.selectedStartTime = selectedDate;
    this.startTimeEmitter.emit(selectedDate)

    this.calcFutureProgress();
  }

  onStopTimeSet(selectedDate: Date) {
    this.selectedStopTime = selectedDate;
    this.endTimeEmitter.emit(selectedDate);

    this.calcFutureProgress();
  }

  calcCurrentProgress() {
    if(this.task!.eta == 0) {
      this.currentProgress = "0%";
    } else {
      const fraction =  this.task!.workTime / this.task!.eta * 100;
      this.currentProgress = fraction + "%";
    }
  }

  calcFutureProgress() {
    //const currentProgressNumber = parseFloat(this.currentProgress.slice(0, -1)) / 100;
    if(this.selectedStartTime != undefined && this.selectedStopTime != undefined) {
      const fraction = this.dateDiffInDays(this.selectedStartTime, this.selectedStopTime) / this.task!.eta * 100
      this.futureProgress = fraction + "%";
    } else {
      this.futureProgress = "0%";
    }


  }

  dateDiffInDays(a: Date, b: Date) {
    const diffMs = b.getTime() - a.getTime();
    return Math.floor((diffMs / 1000) / 60);
  }

  schedule() {
    if(this.scheduleInfo == undefined) {
      console.log("Direct Date" + this.selectedStartTime!.toString())
      console.log("Moment Format" + moment(this.selectedStartTime!).format('YYYY-MM-DDTHH:mm:ss.SSSZ'))
      this.scheduleService.schedulesTaskIDAdvancedPut(this.task!.taskID, {
        scheduleStartTime: moment(this.selectedStartTime!).format('YYYY-MM-DDTHH:mm:ss.SSSZ'),
        scheduleStopTime: moment(this.selectedStopTime!).format('YYYY-MM-DDTHH:mm:ss.SSSZ')
      }).subscribe({
        next: resp => {
          this.onSchedule.emit(resp as AdvancedScheduleInfo);
        }
      })
    } else {
      console.log("Direct Date" + this.selectedStartTime!.toString())
      console.log("Moment Format" + moment(this.selectedStartTime!).format('YYYY-MM-DDTHH:mm:ss.SSSZ'))
      this.scheduleService.schedulesScheduleIDAdvancedPost(this.scheduleInfo.scheduleID!, {
        scheduleStartTime: moment(this.selectedStartTime!).format('YYYY-MM-DDTHH:mm:ss.SSSZ'),
        scheduleStopTime: moment(this.selectedStopTime!).format('YYYY-MM-DDTHH:mm:ss.SSSZ')
      }).subscribe({
        next: resp => {
          this.onSchedule.emit(resp as AdvancedScheduleInfo);
          console.log("Rescheduled")
        }
      })
    }
  }

  addSlideMinutes() {
    if(this.selectedStartTime != undefined) {
      const updatedStopTime = moment(this.selectedStartTime);
      updatedStopTime.add(this.slideMinutes, 'm');

      if(updatedStopTime.isAfter(moment(this.selectedStopTime))) {
        this.setStopTime(updatedStopTime.toDate());
      }
    }
  }
}
